#! /usr/bin/env python3

# Copyright (c) 2008, 2011, 2017 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ovs_build_helpers import soutil
import sys
import getopt
import os


def parse_include_dirs():
    include_dirs = []
    options, args = getopt.gnu_getopt(sys.argv[1:], 'I:', ['include='])
    for key, value in options:
        if key in ['-I', '--include']:
            include_dirs.append(value.split(','))
        else:
            assert False

    include_dirs.append(['.'])
    return include_dirs, args


def find_include_file(include_info, name):
    for info in include_info:
        if len(info) == 2:
            dir = info[1]
            var = "$(%s)/" % info[0]
        else:
            dir = info[0]
            var = ""

        file = "%s/%s" % (dir, name)
        try:
            os.stat(file)
            return var + name
        except OSError:
            pass
    sys.stderr.write("%s not found in: %s\n" %
                     (name, ' '.join(str(include_info))))
    return None


def sodepends(include_info, filenames, dst):
    ok = True
    print("# Generated automatically -- do not modify!    "
          "-*- buffer-read-only: t -*-")
    for toplevel in sorted(filenames):
        # Skip names that don't end in .in.
        if not toplevel.endswith('.in'):
            continue

        # Open file.
        include_dirs = [info[1] if len(info) == 2 else info[0]
                        for info in include_info]
        fn = soutil.find_file(include_dirs, toplevel)
        if not fn:
            ok = False
            continue
        try:
            outer = open(fn)
        except IOError as e:
            sys.stderr.write("%s: open: %s\n" % (fn, e.strerror))
            ok = False
            continue

        dependencies = []
        while True:
            line = outer.readline()
            if not line:
                break

            name = soutil.extract_include_directive(line)
            if name:
                include_file = find_include_file(include_info, name)
                if include_file:
                    dependencies.append(include_file)
                else:
                    ok = False

        dst.write("\n%s:" % toplevel[:-3])
        for s in [toplevel] + sorted(dependencies):
            dst.write(' \\\n\t%s' % s)
        dst.write('\n')
        for s in [toplevel] + sorted(dependencies):
            dst.write('%s:\n' % s)
    return ok


if __name__ == '__main__':
    include_dirs, args = parse_include_dirs()
    error = not sodepends(include_dirs, args, sys.stdout)
    sys.exit(1 if error else 0)



# -*- coding: utf-8 -*-
#
# Open Virtual Network (OVN) documentation build configuration file, created by
# sphinx-quickstart on Fri Sep 30 09:57:36 2016.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import importlib
import string
import sys

use_rtd_theme = importlib.util.find_spec('sphinx_rtd_theme') is not None
if not use_rtd_theme:
    print("Cannot find 'sphinx_rtd_theme' package. "
          "Falling back to default theme.")

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
needs_sphinx = '1.1'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.todo']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'contents'

# General information about the project.
project = u'Open Virtual Network (OVN)'
copyright = u'2020-2023, The Open Virtual Network (OVN) Development Community'
author = u'The Open Virtual Network (OVN) Development Community'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The full version, including alpha/beta/rc tags.
release = None
filename = "../configure.ac"
with open(filename, 'r') as f:
    for line in f:
        if 'AC_INIT' in line:
            # Parse "AC_INIT(openvswitch, 2.7.90, bugs@openvswitch.org)":
            release = line.split(',')[1].strip(string.whitespace + '[]')
            break
if release is None:
    sys.stderr.write('%s: failed to determine Open Virtual Network (OVN) '
                     'version\n' % filename)
    sys.exit(1)

# The short X.Y version.
#
# However, it's important to know the difference between, e.g., 2.7
# and 2.7.90, which can be very different versions (2.7.90 may be much
# closer to 2.8 than to 2.7), so check for that.
version = release if '.90' in release else '.'.join(release.split('.')[0:2])

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# If true, check the validity of #anchors in links.
linkcheck_anchors = False

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
if use_rtd_theme:
    html_theme = 'sphinx_rtd_theme'

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#
html_logo = '_static/logo.png'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
_man_pages = [
    ('ovn-sim.1',
     u'Open Virtual Network simulator environment'),
]

# Generate list of (path, name, description, [author, ...], section)
man_pages = [
    ('ref/%s' % file_name, file_name.split('.', 1)[0],
     description, [author], file_name.split('.', 1)[1])
    for file_name, description in _man_pages]



#!/usr/bin/env python3
import argparse
import string


def strip(val):
    """Strip whitespace and quotation marks from val"""
    return val.strip(f"{string.whitespace}\"'")


def parse_acl_log(line):
    """Convert an ACL log string into a dict"""
    # First cut off the logging preamble.
    # We're assuming the default log format.
    acl_log = {}
    _, _, details = line.rpartition("|")

    # acl_details are things like the acl name, direction,
    # verdict, and severity. packet_details are things like
    # the protocol, addresses, and ports of the packet being
    # logged.
    acl_details, _, packet_details = details.partition(":")
    for datum in acl_details.split(","):
        name, _, value = datum.rpartition("=")
        acl_log[strip(name)] = strip(value)

    for datum in packet_details.split(","):
        name, _, value = datum.rpartition("=")
        if not name:
            # The protocol is not preceded by "protocol="
            # so we need to add it manually.
            name = "protocol"
        acl_log[strip(name)] = strip(value)

    return acl_log


def get_acl_log(entry_num=1):
    with open("ovn-controller.log", "r") as controller_log:
        acl_logs = [line.rstrip() for line in controller_log
                    if "acl_log" in line]
        try:
            return acl_logs[entry_num - 1]
        except IndexError:
            print(
                f"There were not {entry_num} acl_log entries, "
                f"only {len(acl_logs)}"
            )
            exit(1)


def add_parser_args(parser):
    parser.add_argument("--entry-num", type=int, default=1)

    # There are other possible things that can be in an ACL log,
    # and if we need those in the future, we can add them later.
    parser.add_argument("--name")
    parser.add_argument("--verdict")
    parser.add_argument("--severity")
    parser.add_argument("--protocol")
    parser.add_argument("--vlan_tci")
    parser.add_argument("--dl_src")
    parser.add_argument("--dl_dst")
    parser.add_argument("--nw_src")
    parser.add_argument("--nw_dst")
    parser.add_argument("--nw_tos")
    parser.add_argument("--nw_ecn")
    parser.add_argument("--nw_ttl")
    parser.add_argument("--icmp_type")
    parser.add_argument("--icmp_code")
    parser.add_argument("--tp_src")
    parser.add_argument("--tp_dst")
    parser.add_argument("--tcp_flags")
    parser.add_argument("--ipv6_src")
    parser.add_argument("--ipv6_dst")


def main():
    parser = argparse.ArgumentParser()
    add_parser_args(parser)
    args = parser.parse_args()

    acl_log = get_acl_log(args.entry_num)
    parsed_log = parse_acl_log(acl_log)

    # Express command line arguments as a dict, omitting any arguments that
    # were not provided by the user.
    expected = {k: v for k, v in vars(args).items() if v is not None}
    del expected["entry_num"]

    for key, val in expected.items():
        try:
            if parsed_log[key] != val:
                print(
                    f"Expected log {key}={val} but got "
                    f"{key}={parsed_log[key]} "
                    f"in:\n\t'{acl_log}'"
                )
                exit(1)
        except KeyError:
            print(
                f"Expected log {key}={val} but {key} does not exist in:\n"
                f"\t'{acl_log}'"
            )
            exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import time

import ovs.daemon
import ovs.unixctl
import ovs.unixctl.server

import binascii
from socket import *  # noqa: F401,F403
from scapy.all import *  # noqa: F401,F403
from scapy.all import raw


vlog = ovs.vlog.Vlog("scapy-server")
exiting = False


def exit(conn, argv, aux):
    global exiting

    exiting = True
    conn.reply(None)


def process(data):
    start_time = time.perf_counter()
    vlog.info(f"received payload request: {data}")
    try:
        data = data.replace('\n', '')
        return binascii.hexlify(raw(eval(data))).decode()
    except Exception as e:
        vlog.exception(f"failed to process payload request: {e}")
        return ""
    finally:
        total_time = (time.perf_counter() - start_time) * 1000
        vlog.info(f"took {total_time:.2f}ms to process payload request")


def payload(conn, argv, aux):
    try:
        conn.reply(process(argv[0]))
    except Exception as e:
        vlog.exception(f"failed to reply to payload request: {e}")


def main():
    parser = argparse.ArgumentParser(
        description="Scapy-based Frame Payload Generator")
    parser.add_argument("--unixctl", help="UNIXCTL socket location or 'none'.")

    ovs.daemon.add_args(parser)
    ovs.vlog.add_args(parser)
    args = parser.parse_args()
    ovs.daemon.handle_args(args)
    ovs.vlog.handle_args(args)

    ovs.daemon.daemonize_start()
    error, server = ovs.unixctl.server.UnixctlServer.create(args.unixctl)
    if error:
        ovs.util.ovs_fatal(error, "could not create unixctl server at %s"
                           % args.unixctl, vlog)

    ovs.unixctl.command_register("exit", "", 0, 0, exit, None)
    ovs.unixctl.command_register("payload", "", 1, 1, payload, None)
    ovs.daemon.daemonize_complete()

    vlog.info("scapy server ready")

    poller = ovs.poller.Poller()
    while not exiting:
        server.run()
        server.wait(poller)
        if exiting:
            poller.immediate_wake()
        poller.block()
    server.close()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# Copyright (c) 2016, 2017 Red Hat, Inc.
# Copyright (c) 2018 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import email
import getopt
import os
import re
import sys

RETURN_CHECK_INITIAL_STATE = 0
RETURN_CHECK_STATE_WITH_RETURN = 1
RETURN_CHECK_AWAITING_BRACE = 2
EXIT_FAILURE = 1
__errors = 0
__warnings = 0
empty_return_check_state = 0
print_file_name = None
checking_file = False
total_line = 0
colors = False
spellcheck = False
quiet = False
spell_check_dict = None
MAX_LINE_LEN = 79


def open_spell_check_dict():
    import enchant

    try:
        import codespell_lib
        codespell_dir = os.path.dirname(codespell_lib.__file__)
        codespell_file = os.path.join(codespell_dir, 'data', 'dictionary.txt')
        if not os.path.exists(codespell_file):
            codespell_file = ''
    except:
        codespell_file = ''

    try:
        extra_keywords = ['ovs', 'vswitch', 'vswitchd', 'ovs-vswitchd',
                          'netdev', 'selinux', 'ovs-ctl', 'dpctl', 'ofctl',
                          'openvswitch', 'dpdk', 'hugepage', 'hugepages',
                          'pmd', 'upcall', 'vhost', 'rx', 'tx', 'vhostuser',
                          'openflow', 'qsort', 'rxq', 'txq', 'perf', 'stats',
                          'struct', 'int', 'char', 'bool', 'upcalls', 'nicira',
                          'bitmask', 'ipv4', 'ipv6', 'tcp', 'tcp4', 'tcpv4',
                          'udp', 'udp4', 'udpv4', 'icmp', 'icmp4', 'icmpv6',
                          'vlan', 'vxlan', 'cksum', 'csum', 'checksum',
                          'ofproto', 'numa', 'mempool', 'mempools', 'mbuf',
                          'mbufs', 'hmap', 'cmap', 'smap', 'dhcpv4', 'dhcp',
                          'dhcpv6', 'opts', 'metadata', 'geneve', 'mutex',
                          'netdev', 'netdevs', 'subtable', 'virtio', 'qos',
                          'policer', 'datapath', 'tunctl', 'attr', 'ethernet',
                          'ether', 'defrag', 'defragment', 'loopback', 'sflow',
                          'acl', 'initializer', 'recirc', 'xlated', 'unclosed',
                          'netlink', 'msec', 'usec', 'nsec', 'ms', 'us', 'ns',
                          'kilobits', 'kbps', 'kilobytes', 'megabytes', 'mbps',
                          'gigabytes', 'gbps', 'megabits', 'gigabits', 'pkts',
                          'tuple', 'miniflow', 'megaflow', 'conntrack',
                          'vlans', 'vxlans', 'arg', 'tpid', 'xbundle',
                          'xbundles', 'mbundle', 'mbundles', 'netflow',
                          'localnet', 'odp', 'pre', 'dst', 'dest', 'src',
                          'ethertype', 'cvlan', 'ips', 'msg', 'msgs',
                          'liveness', 'userspace', 'eventmask', 'datapaths',
                          'slowpath', 'fastpath', 'multicast', 'unicast',
                          'revalidation', 'namespace', 'qdisc', 'uuid',
                          'ofport', 'subnet', 'revalidation', 'revalidator',
                          'revalidate', 'l2', 'l3', 'l4', 'openssl', 'mtu',
                          'ifindex', 'enum', 'enums', 'http', 'https', 'num',
                          'vconn', 'vconns', 'conn', 'nat', 'memset', 'memcmp',
                          'strcmp', 'strcasecmp', 'tc', 'ufid', 'api',
                          'ofpbuf', 'ofpbufs', 'hashmaps', 'hashmap', 'deref',
                          'dereference', 'hw', 'prio', 'sendmmsg', 'sendmsg',
                          'malloc', 'free', 'alloc', 'pid', 'ppid', 'pgid',
                          'uid', 'gid', 'sid', 'utime', 'stime', 'cutime',
                          'cstime', 'vsize', 'rss', 'rsslim', 'whcan', 'gtime',
                          'eip', 'rip', 'cgtime', 'dbg', 'gw', 'sbrec', 'bfd',
                          'sizeof', 'pmds', 'nic', 'nics', 'hwol', 'encap',
                          'decap', 'tlv', 'tlvs', 'decapsulation', 'fd',
                          'cacheline', 'xlate', 'skiplist', 'idl',
                          'comparator', 'natting', 'alg', 'pasv', 'epasv',
                          'wildcard', 'nated', 'amd64', 'x86_64',
                          'recirculation', 'linux', 'afxdp', 'promisc', 'goto',
                          'misconfigured', 'misconfiguration', 'checkpatch',
                          'debian', 'travis', 'cirrus', 'appveyor', 'faq',
                          'erspan', 'const', 'hotplug', 'addresssanitizer',
                          'ovsdb', 'dpif', 'veth', 'rhel', 'jsonrpc', 'json',
                          'syscall', 'lacp', 'ipf', 'skb', 'valgrind']

        global spell_check_dict

        spell_check_dict = enchant.Dict("en_US")

        if codespell_file:
            with open(codespell_file) as f:
                for line in f.readlines():
                    words = line.strip().split('>')[1].strip(', ').split(',')
                    for word in words:
                        spell_check_dict.add_to_session(word.strip())

        for kw in extra_keywords:
            spell_check_dict.add_to_session(kw)

        return True
    except:
        return False


def get_color_end():
    global colors
    if colors:
        return "\033[00m"
    return ""


def get_red_begin():
    global colors
    if colors:
        return "\033[91m"
    return ""


def get_yellow_begin():
    global colors
    if colors:
        return "\033[93m"
    return ""


def print_error(message):
    global __errors
    print("%sERROR%s: %s" % (get_red_begin(), get_color_end(), message))

    __errors = __errors + 1


def print_warning(message):
    global __warnings
    print("%sWARNING%s: %s" % (get_yellow_begin(), get_color_end(), message))

    __warnings = __warnings + 1


def reset_counters():
    global __errors, __warnings, total_line

    __errors = 0
    __warnings = 0
    total_line = 0


# These are keywords whose names are normally followed by a space and
# something in parentheses (usually an expression) then a left curly brace.
#
# 'do' almost qualifies but it's also used as "do { ... } while (...);".
__parenthesized_constructs = 'if|for|while|switch|[_A-Z]+FOR_*EACH[_A-Z0-9]*'

__regex_added_line = re.compile(r'^\+{1,2}[^\+][\w\W]*')
__regex_subtracted_line = re.compile(r'^\-{1,2}[^\-][\w\W]*')
__regex_leading_with_whitespace_at_all = re.compile(r'^\s+')
__regex_leading_with_spaces = re.compile(r'^ +[\S]+')
__regex_trailing_whitespace = re.compile(r'[^\S]+$')
__regex_single_line_feed = re.compile(r'^\f$')
__regex_for_if_missing_whitespace = re.compile(r' +(%s)[\(]'
                                               % __parenthesized_constructs)
__regex_hash_define_for_each = re.compile(
    r'#define [_A-Z]+FOR_*EACH[_A-Z0-9]*\(')
__regex_for_if_too_much_whitespace = re.compile(r' +(%s)  +[\(]'
                                                % __parenthesized_constructs)
__regex_for_if_parens_whitespace = \
    re.compile(r' +(%s) \( +[\s\S]+\)' % __parenthesized_constructs)
__regex_is_for_if_single_line_bracket = \
    re.compile(r'^ +(%s) \(.*\)' % __parenthesized_constructs)
__regex_ends_with_bracket = \
    re.compile(r'[^\s]\) {(\s+/\*[\s\Sa-zA-Z0-9\.,\?\*/+-]*)?$')
__regex_ptr_declaration_missing_whitespace = re.compile(r'[a-zA-Z0-9]\*[^*]')
__regex_cast_missing_whitespace = re.compile(r'\)[a-zA-Z0-9]')
__regex_is_comment_line = re.compile(r'^\s*(/\*|\*\s)')
__regex_has_comment = re.compile(r'.*(/\*|\*\s)')
__regex_has_c99_comment = re.compile(r'.*//.*$')
__regex_trailing_operator = re.compile(r'^[^ ]* [^ ]*[?:]$')
__regex_conditional_else_bracing = re.compile(r'^\s*else\s*{?$')
__regex_conditional_else_bracing2 = re.compile(r'^\s*}\selse\s*$')
__regex_has_xxx_mark = re.compile(r'.*xxx.*', re.IGNORECASE)
__regex_added_doc_rst = re.compile(
                    r'\ndiff .*Documentation/.*rst\nnew file mode')
__regex_empty_return = re.compile(r'\s*return;')
__regex_if_macros = re.compile(r'^ +(%s) \([\S]([\s\S]+[\S])*\) { +\\' %
                               __parenthesized_constructs)
__regex_nonascii_characters = re.compile("[^\u0000-\u007f]")
__regex_efgrep = re.compile(r'.*[ef]grep.*$')

skip_leading_whitespace_check = False
skip_trailing_whitespace_check = False
skip_gerrit_change_id_check = False
skip_block_whitespace_check = False
skip_signoff_check = False
skip_committer_signoff_check = False

# Don't enforce character limit on files that include these characters in their
# name, as they may have legitimate reasons to have longer lines.
#
# Python isn't checked as flake8 performs these checks during build.
line_length_blacklist = re.compile(
    r'\.(am|at|etc|in|m4|mk|patch|py|yml)$|^debian/.*$')

# Don't enforce a requirement that leading whitespace be all spaces on
# files that include these characters in their name, since these kinds
# of files need lines with leading tabs.
leading_whitespace_blacklist = re.compile(r'\.(mk|am|at)$|^debian/.*$')


def is_subtracted_line(line):
    """Returns TRUE if the line in question has been removed."""
    return __regex_subtracted_line.search(line) is not None


def is_added_line(line):
    """Returns TRUE if the line in question is an added line.
    """
    global checking_file
    return __regex_added_line.search(line) is not None or checking_file


def added_line(line):
    """Returns the line formatted properly by removing diff syntax"""
    global checking_file
    if not checking_file:
        return line[1:]
    return line


def leading_whitespace_is_spaces(line):
    """Returns TRUE if the leading whitespace in added lines is spaces
    """
    if skip_leading_whitespace_check:
        return True
    if (__regex_leading_with_whitespace_at_all.search(line) is not None and
            __regex_single_line_feed.search(line) is None):
        return __regex_leading_with_spaces.search(line) is not None

    return True


def trailing_whitespace_or_crlf(line):
    """Returns TRUE if the trailing characters is whitespace
    """
    if skip_trailing_whitespace_check:
        return False
    return (__regex_trailing_whitespace.search(line) is not None and
            __regex_single_line_feed.search(line) is None)


def if_and_for_whitespace_checks(line):
    """Return TRUE if there is appropriate whitespace after if, for, while
    """
    if skip_block_whitespace_check:
        return True
    if (__regex_for_if_missing_whitespace.search(line) is not None and
        __regex_hash_define_for_each.search(line) is None):
        return False
    if (__regex_for_if_too_much_whitespace.search(line) is not None or
        __regex_for_if_parens_whitespace.search(line)):
        return False
    return True


def if_and_for_end_with_bracket_check(line):
    """Return TRUE if there is a bracket at the end of an if, for, while
       block which fits on a single line ie: 'if (foo)'"""

    def balanced_parens(line):
        """This is a rather naive counter - it won't deal with quotes"""
        balance = 0
        for letter in line:
            if letter == '(':
                balance += 1
            elif letter == ')':
                balance -= 1
        return balance == 0

    if __regex_is_for_if_single_line_bracket.search(line) is not None:
        if not balanced_parens(line):
            return True

        # Early return to avoid potential catastrophic backtracking in the
        # __regex_if_macros regex
        if len(line) == MAX_LINE_LEN - 1 and line[-1] == ')':
            return True

        if __regex_ends_with_bracket.search(line) is None:
            if line.endswith("\\") and \
               __regex_if_macros.match(line) is not None:
                return True
            else:
                return False

    if __regex_conditional_else_bracing.match(line) is not None:
        return False
    if __regex_conditional_else_bracing2.match(line) is not None:
        return False
    return True


def pointer_whitespace_check(line):
    """Return TRUE if there is no space between a pointer name and the
       asterisk that denotes this is a apionter type, ie: 'struct foo*'"""
    return __regex_ptr_declaration_missing_whitespace.search(line) is not None


def nonascii_character_check(line):
    """Return TRUE if inappropriate Unicode characters are detected """
    return __regex_nonascii_characters.search(line) is not None


def cast_whitespace_check(line):
    """Return TRUE if there is no space between the '()' used in a cast and
       the expression whose type is cast, i.e.: '(void *)foo'"""
    return __regex_cast_missing_whitespace.search(line) is not None


def line_length_check(line):
    """Return TRUE if the line length is too long"""
    if len(line) > MAX_LINE_LEN:
        print_warning("Line is %d characters long (recommended limit is %d)"
                      % (len(line), MAX_LINE_LEN))
        return True
    return False


def is_comment_line(line):
    """Returns TRUE if the current line is part of a block comment."""
    return __regex_is_comment_line.match(line) is not None


def has_comment(line):
    """Returns TRUE if the current line contains a comment or is part of
       a block comment."""
    return __regex_has_comment.match(line) is not None


def has_c99_comment(line):
    """Returns TRUE if the current line contains C99 style comment (//)."""
    return __regex_has_c99_comment.match(line) is not None


def trailing_operator(line):
    """Returns TRUE if the current line ends with an operatorsuch as ? or :"""
    return __regex_trailing_operator.match(line) is not None


def has_xxx_mark(line):
    """Returns TRUE if the current line contains 'xxx'."""
    return __regex_has_xxx_mark.match(line) is not None


def has_efgrep(line):
    """Returns TRUE if the current line contains 'egrep' or 'fgrep'."""
    return __regex_efgrep.match(line) is not None


def filter_comments(current_line, keep=False):
    """remove all of the c-style comments in a line"""
    STATE_NORMAL = 0
    STATE_COMMENT_SLASH = 1
    STATE_COMMENT_CONTENTS = 3
    STATE_COMMENT_END_SLASH = 4

    state = STATE_NORMAL
    sanitized_line = ''
    check_state = STATE_NORMAL
    only_whitespace = True

    if keep:
        check_state = STATE_COMMENT_CONTENTS

    for c in current_line:
        if c == '/':
            if state == STATE_NORMAL:
                state = STATE_COMMENT_SLASH
            elif state == STATE_COMMENT_SLASH:
                # This is for c++ style comments.  We will warn later
                return sanitized_line[:1]
            elif state == STATE_COMMENT_END_SLASH:
                c = ''
                state = STATE_NORMAL
        elif c == '*':
            if only_whitespace:
                # just assume this is a continuation from the previous line
                # as a comment
                state = STATE_COMMENT_END_SLASH
            elif state == STATE_COMMENT_SLASH:
                state = STATE_COMMENT_CONTENTS
                sanitized_line = sanitized_line[:-1]
            elif state == STATE_COMMENT_CONTENTS:
                state = STATE_COMMENT_END_SLASH
        elif state == STATE_COMMENT_END_SLASH:
            # Need to re-introduce the star from the previous state, since
            # it may have been clipped by the state check below.
            c = '*' + c
            state = STATE_COMMENT_CONTENTS
        elif state == STATE_COMMENT_SLASH:
            # Need to re-introduce the slash from the previous state, since
            # it may have been clipped by the state check below.
            c = '/' + c
            state = STATE_NORMAL

        if state != check_state:
            c = ''

        if not c.isspace():
            only_whitespace = False

        sanitized_line += c

    return sanitized_line


def check_spelling(line, comment):
    if not spell_check_dict or not spellcheck:
        return False

    if line.startswith('Fixes: '):
        return False

    words = filter_comments(line, True) if comment else line
    words = words.replace(':', ' ').split(' ')

    flagged_words = []
    num_suggestions = 3

    for word in words:
        skip = False
        strword = re.subn(r'\W+', '', word)[0].replace(',', '')
        if (len(strword)
                and not spell_check_dict.check(strword.lower())
                and not spell_check_dict.check(word.lower())):
            if any([check_char in word
                    for check_char in ['=', '(', '-', '_', '/', '\'']]):
                skip = True

            # special case the '.'
            if '.' in word and not word.endswith('.'):
                skip = True

            # skip proper nouns and references to macros
            if strword.isupper() or (strword[0].isupper() and
                                     strword[1:].islower()):
                skip = True

            # skip words containing numbers
            if any(check_char.isdigit() for check_char in strword):
                skip = True

            if not skip:
                flagged_words.append(strword)

    if len(flagged_words) > 0:
        for mistake in flagged_words:
            print_warning("Possible misspelled word: \"%s\"" % mistake)
            print("Did you mean: ",
                  spell_check_dict.suggest(mistake)[:num_suggestions])

        return True

    return False


def __check_doc_is_listed(text, doctype, docdir, docfile):
    if doctype == 'rst':
        beginre = re.compile(r'\+\+\+.*{}/index.rst'.format(docdir))
        docre = re.compile(r'\n\+.*{}'.format(docfile.replace('.rst', '')))
    elif doctype == 'automake':
        beginre = re.compile(r'\+\+\+.*Documentation/automake.mk')
        docre = re.compile(r'\n\+\t{}/{}'.format(docdir, docfile))
    else:
        raise NotImplementedError("Invalid doctype: {}".format(doctype))

    res = beginre.search(text)
    if res is None:
        return True

    hunkstart = res.span()[1]
    hunkre = re.compile(r'\n(---|\+\+\+) (\S+)')
    res = hunkre.search(text[hunkstart:])
    if res is None:
        hunkend = len(text)
    else:
        hunkend = hunkstart + res.span()[0]

    hunk = text[hunkstart:hunkend]
    # find if the file is being added.
    if docre.search(hunk) is not None:
        return False

    return True


def __check_new_docs(text, doctype):
    """Check if the documentation is listed properly. If doctype is 'rst' then
       the index.rst is checked. If the doctype is 'automake' then automake.mk
       is checked. Returns TRUE if the new file is not listed."""
    failed = False
    new_docs = __regex_added_doc_rst.findall(text)
    for doc in new_docs:
        docpathname = doc.split(' ')[2]
        gitdocdir, docfile = os.path.split(docpathname.rstrip('\n'))
        if docfile == "index.rst":
            continue

        if gitdocdir.startswith('a/'):
            docdir = gitdocdir.replace('a/', '', 1)
        else:
            docdir = gitdocdir

        if __check_doc_is_listed(text, doctype, docdir, docfile):
            if doctype == 'rst':
                print_warning("New doc {} not listed in {}/index.rst".format(
                              docfile, docdir))
            elif doctype == 'automake':
                print_warning("New doc {} not listed in "
                              "Documentation/automake.mk".format(docfile))
            else:
                raise NotImplementedError("Invalid doctype: {}".format(
                                          doctype))

            failed = True

    return failed


def check_doc_docs_automake(text):
    return __check_new_docs(text, 'automake')


def check_new_docs_index(text):
    return __check_new_docs(text, 'rst')


def empty_return_with_brace(line):
    """Returns TRUE if a function contains a return; followed
       by one or more line feeds and terminates with a '}'
       at start of line"""

    def empty_return(line):
        """Returns TRUE if a function has a 'return;'"""
        return __regex_empty_return.match(line) is not None

    global empty_return_check_state
    if empty_return_check_state == RETURN_CHECK_INITIAL_STATE \
       and empty_return(line):
        empty_return_check_state = RETURN_CHECK_STATE_WITH_RETURN
    elif empty_return_check_state == RETURN_CHECK_STATE_WITH_RETURN \
         and (re.match(r'^}$', line) or len(line) == 0):
        if re.match('^}$', line):
            empty_return_check_state = RETURN_CHECK_AWAITING_BRACE
    else:
        empty_return_check_state = RETURN_CHECK_INITIAL_STATE

    if empty_return_check_state == RETURN_CHECK_AWAITING_BRACE:
        empty_return_check_state = RETURN_CHECK_INITIAL_STATE
        return True

    return False


file_checks = [
        {'regex': __regex_added_doc_rst,
         'check': check_new_docs_index},
        {'regex': __regex_added_doc_rst,
         'check': check_doc_docs_automake}
]

checks = [
    {'regex': None,
     'match_name': lambda x: not line_length_blacklist.search(x),
     'check': lambda x: line_length_check(x)},

    {'regex': None,
     'match_name': lambda x: not leading_whitespace_blacklist.search(x),
     'check': lambda x: not leading_whitespace_is_spaces(x),
     'print': lambda: print_warning("Line has non-spaces leading whitespace")},

    {'regex': None, 'match_name': None,
     'check': lambda x: trailing_whitespace_or_crlf(x),
     'print': lambda: print_warning("Line has trailing whitespace")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: not if_and_for_whitespace_checks(x),
     'print': lambda: print_error("Improper whitespace around control block")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: not if_and_for_end_with_bracket_check(x),
     'print': lambda: print_error("Inappropriate bracing around statement")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: pointer_whitespace_check(x),
     'print':
     lambda: print_error("Inappropriate spacing in pointer declaration")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'check': lambda x: nonascii_character_check(x),
     'print':
     lambda: print_error("Inappropriate non-ascii characters detected.")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: cast_whitespace_check(x),
     'print':
     lambda: print_error("Inappropriate spacing around cast")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: trailing_operator(x),
     'print':
     lambda: print_error("Line has '?' or ':' operator at end of line")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: has_comment(x),
     'check': lambda x: has_xxx_mark(x),
     'print': lambda: print_warning("Comment with 'xxx' marker")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': lambda x: has_c99_comment(x),
     'print': lambda: print_error("C99 style comment")},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: has_comment(x),
     'check': lambda x: check_spelling(x, True)},

    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'check': lambda x: empty_return_with_brace(x),
     'interim_line': True,
     'print':
     lambda: print_warning("Empty return followed by brace, consider omitting")
     },

    {'regex': r'(\.at|\.sh)$', 'match_name': None,
     'check': lambda x: has_efgrep(x),
     'print':
     lambda: print_error("grep -E/-F should be used instead of egrep/fgrep")},
]


def regex_function_factory(func_name):
    regex = re.compile(r'\b%s\([^)]*\)' % func_name)
    return lambda x: regex.search(x) is not None


def regex_error_factory(description):
    return lambda: print_error(description)


def regex_warn_factory(description):
    return lambda: print_warning(description)


std_functions = [
        ('malloc', 'Use xmalloc() in place of malloc()'),
        ('calloc', 'Use xcalloc() in place of calloc()'),
        ('realloc', 'Use xrealloc() in place of realloc()'),
        ('strdup', 'Use xstrdup() in place of strdup()'),
        ('asprintf', 'Use xasprintf() in place of asprintf()'),
        ('vasprintf', 'Use xvasprintf() in place of vasprintf()'),
        ('strcpy', 'Use ovs_strlcpy() in place of strcpy()'),
        ('strlcpy', 'Use ovs_strlcpy() in place of strlcpy()'),
        ('strncpy', 'Use ovs_strzcpy() in place of strncpy()'),
        ('strerror', 'Use ovs_strerror() in place of strerror()'),
        ('sleep', 'Use xsleep() in place of sleep()'),
        ('abort', 'Use ovs_abort() in place of abort()'),
        ('assert', 'Use ovs_assert() in place of assert()'),
        ('error', 'Use ovs_error() in place of error()'),
]

checks += [
    {'regex': r'(\.c|\.h)(\.in)?$',
     'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': regex_function_factory(function_name),
     'print': regex_error_factory(description)}
    for (function_name, description) in std_functions]

easy_to_misuse_api = [
        ('ovsrcu_barrier',
            'lib/ovs-rcu.c',
            'Are you sure you need to use ovsrcu_barrier(), '
            'in most cases ovsrcu_synchronize() will be fine?'),
        ]

checks += [
    {'regex': r'(\.c)(\.in)?$',
     'match_name': lambda x: x != location,
     'prereq': lambda x: not is_comment_line(x),
     'check': regex_function_factory(function_name),
     'print': regex_warn_factory(description)}
    for (function_name, location, description) in easy_to_misuse_api]


def regex_operator_factory(operator):
    regex = re.compile(r'^[^#][^"\']*[^ "]%s[^ "\'][^"]*' % operator)
    return lambda x: regex.search(filter_comments(x)) is not None


infix_operators = \
    [re.escape(op) for op in ['%', '<<', '>>', '<=', '>=', '==', '!=',
            '^', '|', '&&', '||', '?:', '=', '+=', '-=', '*=', '/=', '%=',
            '&=', '^=', '|=', '<<=', '>>=']] \
    + [r'[^<" ]<[^=" ]',
       r'[^\->" ]>[^=" ]',
       r'[^ !()/"]\*[^/]',
       r'[^ !&()"]&',
       r'[^" +(]\+[^"+;]',
       r'[^" \-(]\-[^"\->;]',
       r'[^" <>=!^|+\-*/%&]=[^"=]',
       r'[^* ]/[^* ]']
checks += [
    {'regex': r'(\.c|\.h)(\.in)?$', 'match_name': None,
     'prereq': lambda x: not is_comment_line(x),
     'check': regex_operator_factory(operator),
     'print': lambda: print_warning("Line lacks whitespace around operator")}
    for operator in infix_operators]


def get_file_type_checks(filename):
    """Returns the list of checks for a file based on matching the filename
       against regex."""
    global checks
    checkList = []
    for check in checks:
        regex_check = True
        match_check = True

        if check['regex'] is None and check['match_name'] is None:
            checkList.append(check)
            continue

        if check['regex'] is not None and \
           re.compile(check['regex']).search(filename) is None:
            regex_check = False

        if check['match_name'] is not None and \
                not check['match_name'](filename):
            match_check = False

        if regex_check and match_check:
            checkList.append(check)
    return checkList


def run_checks(current_file, line, lineno):
    """Runs the various checks for the particular line.  This will take
       filename into account."""
    global checking_file, total_line
    print_line = False
    for check in get_file_type_checks(current_file):
        if 'prereq' in check and not check['prereq'](line):
            continue
        if check['check'](line):
            if 'print' in check:
                check['print']()
            print_line = True

    if print_line:
        if checking_file:
            print("%s:%d:" % (current_file, lineno))
        else:
            print("#%d FILE: %s:%d:" % (total_line, current_file, lineno))
        print("%s\n" % line)


def interim_line_check(current_file, line, lineno):
    """Runs the various checks for the particular interim line.  This will
       take filename into account, and will check for the 'interim_line'
       key before running the check."""
    global checking_file, total_line
    print_line = False
    for check in get_file_type_checks(current_file):
        if 'prereq' in check and not check['prereq'](line):
            continue
        if 'interim_line' in check and check['interim_line']:
            if check['check'](line):
                if 'print' in check:
                    check['print']()
                    print_line = True

    if print_line:
        if checking_file:
            print("%s:%d:" % (current_file, lineno))
        else:
            print("#%d FILE: %s:%d:" % (total_line, current_file, lineno))
        print("%s\n" % line)


def run_file_checks(text):
    """Runs the various checks for the text."""
    for check in file_checks:
        if check['regex'].search(text) is not None:
            check['check'](text)


def run_subject_checks(subject, spellcheck=False):
    warnings = False

    if spellcheck and check_spelling(subject, False):
        warnings = True

    summary = subject[subject.rindex(': ') + 2:]
    area_summary = subject[subject.index(': ') + 2:]
    area_summary_len = len(area_summary)
    if area_summary_len > 70:
        print_warning("The subject, '<area>: <summary>', is over 70 "
                      "characters, i.e., %u." % area_summary_len)
        warnings = True

    if summary[0].isalpha() and summary[0].islower():
        print_warning(
            "The subject summary should start with a capital.")
        warnings = True

    if subject[-1] not in [".", "?", "!"]:
        print_warning(
            "The subject summary should end with a dot.")
        warnings = True

    if warnings:
        print(subject)

    return warnings


def ovs_checkpatch_parse(text, filename, author=None, committer=None):
    global print_file_name, total_line, checking_file, \
        empty_return_check_state

    PARSE_STATE_HEADING = 0
    PARSE_STATE_DIFF_HEADER = 1
    PARSE_STATE_CHANGE_BODY = 2

    lineno = 0
    signatures = []
    co_authors = []
    parse = 0
    current_file = filename if checking_file else ''
    previous_file = ''
    seppatch = re.compile(r'^---([\w]*| \S+)$')
    hunks = re.compile(r'^(---|\+\+\+) (\S+)')
    hunk_differences = re.compile(
        r'^@@ ([0-9-+]+),([0-9-+]+) ([0-9-+]+),([0-9-+]+) @@')
    is_author = re.compile(r'^(Author|From): (.*)$', re.I | re.M | re.S)
    is_committer = re.compile(r'^(Commit: )(.*)$', re.I | re.M | re.S)
    is_subject = re.compile(r'^(Subject: )(.*)$', re.I | re.M | re.S)
    is_signature = re.compile(r'^(Signed-off-by: )(.*)$',
                              re.I | re.M | re.S)
    is_co_author = re.compile(r'^(Co-authored-by: )(.*)$',
                              re.I | re.M | re.S)
    is_gerrit_change_id = re.compile(r'(\s*(change-id: )(.*))$',
                                     re.I | re.M | re.S)
    is_fixes = re.compile(r'(\s*(Fixes:)(.*))$', re.I | re.M | re.S)
    is_fixes_exact = re.compile(r'^Fixes: [0-9a-f]{12} \(".*"\)$')

    tags_typos = {
        r'^Acked by:': 'Acked-by:',
        r'^Reported at:': 'Reported-at:',
        r'^Reported by:': 'Reported-by:',
        r'^Requested by:': 'Requested-by:',
        r'^Reviewed by:': 'Reviewed-by:',
        r'^Submitted at:': 'Submitted-at:',
        r'^Suggested by:': 'Suggested-by:',
    }

    reset_counters()

    for line in text.split("\n"):
        if current_file != previous_file:
            previous_file = current_file

        lineno = lineno + 1
        total_line = total_line + 1

        if line == "\f":
            # Form feed
            continue
        if len(line) <= 0:
            continue

        if checking_file:
            parse = PARSE_STATE_CHANGE_BODY

        if parse == PARSE_STATE_DIFF_HEADER:
            match = hunks.match(line)
            if match:
                parse = PARSE_STATE_CHANGE_BODY
                current_file = match.group(2)[2:]
                print_file_name = current_file
            continue
        elif parse == PARSE_STATE_HEADING:
            if seppatch.match(line):
                parse = PARSE_STATE_DIFF_HEADER
                if not skip_signoff_check:

                    # Check that the patch has an author, that the
                    # author is not among the co-authors, and that the
                    # co-authors are unique.
                    if not author:
                        print_error("Patch lacks author.")
                        continue
                    if " via " in author or "@openvswitch.org" in author:
                        print_error("Author should not be mailing list.")
                        continue
                    if author in co_authors:
                        print_error("Author should not be also be co-author.")
                        continue
                    if len(set(co_authors)) != len(co_authors):
                        print_error("Duplicate co-author.")

                    # Check that the author, all co-authors, and the
                    # committer (if any) signed off.
                    if author not in signatures:
                        print_error("Author %s needs to sign off." % author)
                    for ca in co_authors:
                        if ca not in signatures:
                            print_error("Co-author %s needs to sign off." % ca)
                            break
                    if (committer
                        and author != committer
                        and committer not in signatures
                        and not skip_committer_signoff_check):
                        print_error("Committer %s needs to sign off."
                                    % committer)

                    # Check for signatures that we do not expect.
                    # This is only a warning because there can be,
                    # rarely, a signature chain.
                    #
                    # If we don't have a known committer, and there is
                    # a single extra sign-off, then do not warn
                    # because that extra sign-off is probably the
                    # committer.
                    extra_sigs = [x for x in signatures
                                  if x not in co_authors
                                  and x != author
                                  and x != committer]
                    if len(extra_sigs) > 1 or (committer and extra_sigs):
                        print_warning("Unexpected sign-offs from developers "
                                      "who are not authors or co-authors or "
                                      "committers: %s"
                                      % ", ".join(extra_sigs))
            elif is_committer.match(line):
                committer = is_committer.match(line).group(2)
            elif is_author.match(line):
                author = is_author.match(line).group(2)
            elif is_subject.match(line):
                run_subject_checks(line, spellcheck)
            elif is_signature.match(line):
                m = is_signature.match(line)
                signatures.append(m.group(2))
            elif is_co_author.match(line):
                m = is_co_author.match(line)
                co_authors.append(m.group(2))
            elif (is_gerrit_change_id.match(line) and
                  not skip_gerrit_change_id_check):
                print_error(
                    "Remove Gerrit Change-Id's before submitting upstream.")
                print("%d: %s\n" % (lineno, line))
            elif is_fixes.match(line) and not is_fixes_exact.match(line):
                print_error('"Fixes" tag is malformed.\n'
                            'Use the following format:\n'
                            '  git log -1 '
                            '--pretty=format:"Fixes: %h (\\\"%s\\\")" '
                            '--abbrev=12 COMMIT_REF\n')
                print("%d: %s\n" % (lineno, line))
            elif spellcheck:
                check_spelling(line, False)
            for typo, correct in tags_typos.items():
                m = re.match(typo, line, re.I)
                if m:
                    print_error("%s tag is malformed." % (correct[:-1]))
                    print("%d: %s\n" % (lineno, line))

        elif parse == PARSE_STATE_CHANGE_BODY:
            newfile = hunks.match(line)
            if newfile:
                current_file = newfile.group(2)[2:]
                print_file_name = current_file
                continue
            reset_line_number = hunk_differences.match(line)
            if reset_line_number:
                empty_return_check_state = RETURN_CHECK_INITIAL_STATE
                lineno = int(reset_line_number.group(3))
                if lineno < 0:
                    lineno = -1 * lineno
                lineno -= 1

            if is_subtracted_line(line):
                lineno -= 1
                continue

            cmp_line = added_line(line)

            if not is_added_line(line):
                interim_line_check(current_file, cmp_line, lineno)
                continue

            # Skip files which have /datapath in them, since they are
            # linux or windows coding standards
            if current_file.startswith('datapath'):
                continue
            if current_file.startswith('include/linux'):
                continue
            # "sparse" includes could be copy-pasted from different sources
            # like DPDK or Linux and could contain workarounds not suitable
            # for a common style.
            if current_file.startswith('include/sparse'):
                continue
            if current_file.startswith('utilities/bugtool'):
                continue
            run_checks(current_file, cmp_line, lineno)

    run_file_checks(text)
    if __errors or __warnings:
        return EXIT_FAILURE
    return 0


def usage():
    print("""\
Open vSwitch checkpatch.py
Checks a patch for trivial mistakes.
usage:
%s [options] [PATCH1 [PATCH2 ...] | -f SOURCE1 [SOURCE2 ...] | -1 | -2 | ...]

Input options:
-f|--check-file                Arguments are source files, not patches.
-1, -2, ...                    Check recent commits in this repo.

Check options:
-h|--help                      This help message
-b|--skip-block-whitespace     Skips the if/while/for whitespace tests
-l|--skip-leading-whitespace   Skips the leading whitespace test
-q|--quiet                     Only print error and warning information
-s|--skip-signoff-lines        Tolerate missing Signed-off-by line
-S|--spellcheck                Check C comments and commit-message for possible
                               spelling mistakes
-t|--skip-trailing-whitespace  Skips the trailing whitespace test
   --skip-gerrit-change-id     Skips the gerrit change id test
   --skip-committer-signoff    Skips the committer sign-off test"""
          % sys.argv[0])


def ovs_checkpatch_print_result():
    global quiet, __warnings, __errors, total_line

    if __errors or __warnings:
        print("Lines checked: %d, Warnings: %d, Errors: %d\n" %
              (total_line, __warnings, __errors))
    elif not quiet:
        print("Lines checked: %d, no obvious problems found\n" % (total_line))


def ovs_checkpatch_file(filename):
    try:
        mail = email.message_from_file(open(filename, 'r', encoding='utf8'))
    except:
        print_error("Unable to parse file '%s'. Is it a patch?" % filename)
        return -1

    for part in mail.walk():
        if part.get_content_maintype() == 'multipart':
            continue
    result = ovs_checkpatch_parse(part.get_payload(decode=False), filename,
                                  mail.get('Author', mail['From']),
                                  mail['Commit'])

    if not mail['Subject'] or not mail['Subject'].strip():
        if mail['Subject']:
            mail.replace_header('Subject', sys.argv[-1])
        else:
            mail.add_header('Subject', sys.argv[-1])

        print("Subject missing! Your provisional subject is",
              mail['Subject'])

    if run_subject_checks('Subject: ' + mail['Subject'], spellcheck):
        result = True

    ovs_checkpatch_print_result()
    return result


def partition(pred, iterable):
    """Returns [[trues], [falses]], where [trues] is the items in
    'iterable' that satisfy 'pred' and [falses] is all the rest."""
    trues = []
    falses = []
    for item in iterable:
        if pred(item):
            trues.append(item)
        else:
            falses.append(item)
    return trues, falses


if __name__ == '__main__':
    try:
        numeric_options, args = partition(lambda s: re.match('-[0-9]+$', s),
                                          sys.argv[1:])
        n_patches = int(numeric_options[-1][1:]) if numeric_options else 0

        optlist, args = getopt.getopt(args, 'bhlstfSq',
                                      ["check-file",
                                       "help",
                                       "skip-block-whitespace",
                                       "skip-leading-whitespace",
                                       "skip-signoff-lines",
                                       "skip-trailing-whitespace",
                                       "skip-gerrit-change-id",
                                       "skip-committer-signoff",
                                       "spellcheck",
                                       "quiet"])
    except:
        print("Unknown option encountered. Please rerun with -h for help.")
        sys.exit(EXIT_FAILURE)

    for o, a in optlist:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("-b", "--skip-block-whitespace"):
            skip_block_whitespace_check = True
        elif o in ("-l", "--skip-leading-whitespace"):
            skip_leading_whitespace_check = True
        elif o in ("-s", "--skip-signoff-lines"):
            skip_signoff_check = True
        elif o in ("-t", "--skip-trailing-whitespace"):
            skip_trailing_whitespace_check = True
        elif o in ("--skip-gerrit-change-id"):
            skip_gerrit_change_id_check = True
        elif o in ("--skip-committer-signoff"):
            skip_committer_signoff_check = True
        elif o in ("-f", "--check-file"):
            checking_file = True
        elif o in ("-S", "--spellcheck"):
            if not open_spell_check_dict():
                print("WARNING: The enchant library isn't available.")
                print("         Please install python enchant.")
            else:
                spellcheck = True
        elif o in ("-q", "--quiet"):
            quiet = True
        else:
            print("Unknown option '%s'" % o)
            sys.exit(EXIT_FAILURE)

    if sys.stdout.isatty():
        colors = True

    if n_patches:
        status = 0

        git_log = 'git log --no-color --no-merges --pretty=format:"%H %s" '
        with os.popen(git_log + '-%d' % n_patches, 'r') as f:
            commits = f.read().split("\n")

        for i in reversed(range(0, n_patches)):
            revision, name = commits[i].split(" ", 1)
            f = os.popen('''git format-patch -1 --stdout --pretty=format:"\
Author: %an <%ae>
Commit: %cn <%ce>
Subject: %s

%b" ''' + revision, 'r')
            patch = f.read()
            f.close()

            if not quiet:
                print('== Checking %s ("%s") ==' % (revision[0:12], name))
            result = ovs_checkpatch_parse(patch, revision)
            ovs_checkpatch_print_result()
            if result:
                status = EXIT_FAILURE
        sys.exit(status)

    if not args:
        if sys.stdin.isatty():
            usage()
            sys.exit(EXIT_FAILURE)
        result = ovs_checkpatch_parse(sys.stdin.read(), '-')
        ovs_checkpatch_print_result()
        sys.exit(result)

    status = 0
    for filename in args:
        if not quiet:
            print('== Checking "%s" ==' % filename)
        result = ovs_checkpatch_file(filename)
        if result:
            status = EXIT_FAILURE
    sys.exit(status)



#!/usr/bin/env python3
# Copyright (c) 2020 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Simple python script which connects to tcp server and then
# resets the connection.
import argparse
import socket
import struct
import time

parser = argparse.ArgumentParser(description='')
parser.add_argument("--src-port", type=int, default=11337,
                    help="source port to use")
parser.add_argument("--dst-port", type=int, help="dst port to use")
parser.add_argument("--dst-ip", help="server ip to use")
args = parser.parse_args()
sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
server_address = (args.dst_ip, args.dst_port)
sock.bind(('0.0.0.0', args.src_port))
sock.connect(server_address)
l_onoff = 1
l_linger = 0
time.sleep(1)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER,
                struct.pack('ii', l_onoff, l_linger))
sock.close()
